@@ -10,7 +10,6 @@ from ..lib.objects import *
 from ..lib.async_sessions import AsyncSession as Session
 from .acm import AsyncAcm as Acm
 
-
 class AsyncSubClient(Acm, Session):
     def __init__(self, comId: str, proxies: dict = None, acm: bool = False):
         self.proxies = proxies
@@ -127,14 +126,28 @@ class AsyncSubClient(Acm, Session):
         )
         return UserProfileList(req["userProfileList"]).UserProfileList
 
-    async def get_public_chats(
-        self, filterType: str = "recommended", start: int = 0, size: int = 50
-    ):
-        req = await self.getRequest(
-            f"/x{self.comId}/s/chat/thread?type=public-all&filterType={filterType}&start={start}&size={size}"
-        )
+    async def get_public_chats(self, filterType: str = "recommended", start: int = 0, size: int = 50):
+        req = await self.getRequest(f"/x{self.comId}/s/chat/thread?type=public-all&filterType={filterType}&start={start}&size={size}")
         return ThreadList(req["threadList"]).ThreadList
 
+    async def full_embed(self, link: str, image: BinaryIO, message: str, chatId: str):
+        data = {
+            "type": 0,
+            "content": message,
+            "extensions": {
+                "linkSnippetList": [{
+                    "link": link,
+                    "mediaType": 100,
+                    "mediaUploadValue": b64encode(image.read()).decode(),
+                    "mediaUploadValueContentType": "image/png"
+                }]
+            },
+            "clientRefId": int(timestamp() / 10 % 100000000),
+            "timestamp": int(timestamp() * 1000),
+            "attachedObject": None
+        }
+        return Json(await self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}/message", data))
+
     async def send_message(
         self,
         chatId: str,
@@ -1124,7 +1137,7 @@ class AsyncSubClient(Acm, Session):
 
     async def send_active_time(self, tz: int = int(-timezones // 1000), timers: list = None):
         data = {
-            "userActiveTimeChunkList": [timers if timers else {"start": int(timestamp()), "end": int(timestamp() + 300)}],
+            "userActiveTimeChunkList": timers if timers else [{"start": int(timestamp()), "end": int(timestamp() + 300)}],
             "timestamp": int(timestamp() * 1000),
             "optInAdsFlags": 2147483647,
             "timezone": tz,
