@@ -6,13 +6,13 @@ from sys import _getframe as getframe
 from typing import Union
 
 import ujson as json
-import websocket
 import asyncio
 
+from .websocket_async import WebSocketApp
+
 from ..lib import *
 from ..lib.objects import *
 from .bot import AsyncBot as Bot
-from easy_events import AsyncEvents
 
 try:
     asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
@@ -31,6 +31,9 @@ class AsyncCallbacks(Bot):
 
         self.methods = {
             10: self._resolve_payload,
+            201: self._resolve_channel,
+            304: self._resolve_chat_action_start,
+            306: self._resolve_chat_action_end,
             400: self._resolve_topics,
             1000: self._resolve_chat_message,
         }
@@ -89,11 +92,18 @@ class AsyncCallbacks(Bot):
         }
 
         self.notif_methods = {
+            "18": self.on_alert,
             "53": self.on_member_set_you_host,
             "67": self.on_member_set_you_cohost,
             "68": self.on_member_remove_you_cohost,
         }
 
+        self.chat_action_methods = {
+            "fetch-channel": self.on_fetch_channel,
+            "Typing-start": self.on_user_typing_start,
+            "Typing-end": self.on_user_typing_end,
+        }
+
         self.topics = {
             "online-members": self.on_online_users_update,
             "users-start-typing-at": self.on_user_typing_start,
@@ -102,6 +112,18 @@ class AsyncCallbacks(Bot):
             "users-end-recording-at": self.on_voice_chat_end,
         }
 
+    async def _resolve_chat_action_start(self, data):
+        key = data['o'].get('actions', 0)+"-start"
+        return await self.chat_action_methods.get(key, self.default)(data)
+
+    async def _resolve_chat_action_end(self, data):
+        key = data['o'].get('actions', 0)+"-end"
+        return await self.chat_action_methods.get(key, self.default)(data)
+
+    async def _resolve_channel(self, data):
+            if data['t'] == 201:
+                return await self.chat_action_methods.get("fetch-channel")(data)
+
     async def _resolve_payload(self, data):
         key = f"{data['o']['payload']['notifType']}"
         return await self.notif_methods.get(key)(data)
@@ -134,7 +156,7 @@ class AsyncCallbacks(Bot):
         return registerHandler
 
     async def setCall(self, name, data):
-        await self.call(name, Event(data["o"]).Event)
+        return await self.call(name, Event(data["o"]).Event)
 
     async def on_text_message(self, data):
         if self.is_bot:
@@ -142,68 +164,78 @@ class AsyncCallbacks(Bot):
             new_data = self.build_parameters(new_data)
             await self.trigger(new_data, str_only=True)
 
-        await self.setCall(getframe(0).f_code.co_name, data)
+        return await self.setCall(getframe(0).f_code.co_name, data)
 
-    async def on_member_set_you_host(self, data): self.call(getframe(0).f_code.co_name, Payload(data["o"]).Payload)
-    async def on_member_remove_you_cohost(self, data): self.call(getframe(0).f_code.co_name, Payload(data["o"]).Payload)
-    async def on_member_set_you_cohost(self, data): self.call(getframe(0).f_code.co_name, Payload(data["o"]).Payload)
+    async def on_alert(self, data):
+        if self.is_bot and False:
+            new_data = Payload(data["o"]["payload"]).Payload
+            new_data = self.build_alert_parameters(new_data)
+            await self.trigger(new_data, str_only=True)
 
-    async def on_image_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_youtube_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_strike_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_sticker_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def TYPE_USER_SHARE_EXURL(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def TYPE_USER_SHARE_USER(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_chat_not_answered(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_chat_not_cancelled(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_chat_not_declined(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_video_chat_not_answered(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_video_chat_not_cancelled(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_video_chat_not_declined(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_avatar_chat_not_answered(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_avatar_chat_not_cancelled(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_avatar_chat_not_declined(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_delete_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_group_member_join(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_group_member_leave(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_invite(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_background_changed(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_title_changed(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_icon_changed(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_chat_start(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_video_chat_start(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_avatar_chat_start(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_chat_end(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_video_chat_end(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_avatar_chat_end(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_content_changed(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_screen_room_start(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_screen_room_end(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_host_transfered(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_text_message_force_removed(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_removed_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_text_message_removed_by_admin(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_tip(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_pin_announcement(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_chat_permission_open_to_everyone(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_chat_permission_invited_and_requested(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_voice_chat_permission_invite_only(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_view_only_enabled(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_view_only_disabled(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_unpin_announcement(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_tipping_enabled(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_chat_tipping_disabled(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_timestamp_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
+        return await self.call(getframe(0).f_code.co_name, Payload(data["o"]["payload"]).Payload)
 
-    async def on_welcome_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
-    async def on_invite_message(self, data): await self.setCall(getframe(0).f_code.co_name, data)
 
-    async def on_user_typing_start(self, data): await self.call(getframe(0).f_code.co_name, UsersActions(data).UsersActions)
-    async def on_user_typing_end(self, data): await self.call(getframe(0).f_code.co_name, UsersActions(data).UsersActions)
-    async def on_online_users_update(self, data): await self.call(getframe(0).f_code.co_name, UsersActions(data).UsersActions)
+    async def on_member_set_you_host(self, data): return await self.call(getframe(0).f_code.co_name, Payload(data["o"]["payload"]).Payload)
+    async def on_member_remove_you_cohost(self, data): return await self.call(getframe(0).f_code.co_name, Payload(data["o"]["payload"]).Payload)
+    async def on_member_set_you_cohost(self, data): return await self.call(getframe(0).f_code.co_name, Payload(data["o"]["payload"]).Payload)
+    async def on_fetch_channel(self, data): return await self.call(getframe(0).f_code.co_name, Payload(data["o"]["payload"]).Payload)
 
-    async def default(self, data): await self.call(getframe(0).f_code.co_name, data)
+    async def on_image_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_youtube_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_strike_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_sticker_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def TYPE_USER_SHARE_EXURL(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def TYPE_USER_SHARE_USER(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_chat_not_answered(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_chat_not_cancelled(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_chat_not_declined(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_video_chat_not_answered(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_video_chat_not_cancelled(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_video_chat_not_declined(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_avatar_chat_not_answered(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_avatar_chat_not_cancelled(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_avatar_chat_not_declined(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_delete_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_group_member_join(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_group_member_leave(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_invite(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_background_changed(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_title_changed(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_icon_changed(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_chat_start(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_video_chat_start(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_avatar_chat_start(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_chat_end(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_video_chat_end(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_avatar_chat_end(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_content_changed(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_screen_room_start(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_screen_room_end(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_host_transfered(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_text_message_force_removed(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_removed_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_text_message_removed_by_admin(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_tip(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_pin_announcement(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_chat_permission_open_to_everyone(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_chat_permission_invited_and_requested(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_voice_chat_permission_invite_only(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_view_only_enabled(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_view_only_disabled(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_unpin_announcement(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_tipping_enabled(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_chat_tipping_disabled(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_timestamp_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+
+    async def on_welcome_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+    async def on_invite_message(self, data): return await self.setCall(getframe(0).f_code.co_name, data)
+
+    async def on_user_typing_start(self, data): return await self.call(getframe(0).f_code.co_name, UsersActions(data).UsersActions)
+    async def on_user_typing_end(self, data): return await self.call(getframe(0).f_code.co_name, UsersActions(data).UsersActions)
+    async def on_online_users_update(self, data): return await self.call(getframe(0).f_code.co_name, UsersActions(data).UsersActions)
+
+    async def default(self, data): return await self.call(getframe(0).f_code.co_name, data)
 
 
 class SetAction:
@@ -389,7 +421,7 @@ class WssClient:
             },
             "t": 112,
         }
-        asyncio.sleep(2.2)
+        await asyncio.sleep(2.2)
         await self.wss.send(data)
 
     async def joinVideoChat(self, comId: str, chatId: str, joinType: int = 1):
@@ -411,7 +443,7 @@ class WssClient:
             },
             "t": 108,
         }
-        asyncio.sleep(2.2)
+        await asyncio.sleep(2.2)
         await self.wss.send(data)
 
     async def startVoiceChat(self, comId, chatId: str, joinType: int = 1):
@@ -432,7 +464,7 @@ class WssClient:
             },
             "t": 112,
         }
-        asyncio.sleep(2.2)
+        await asyncio.sleep(2.2)
         await self.wss.send(data)
         data = {
             "o": {
@@ -443,7 +475,7 @@ class WssClient:
             },
             "t": 108,
         }
-        asyncio.sleep(2.2)
+        await asyncio.sleep(2.2)
         await self.wss.send(data)
 
     async def endVoiceChat(self, comId: str, chatId: str, leaveType: int = 2):
@@ -464,7 +496,7 @@ class WssClient:
             },
             "t": 112,
         }
-        asyncio.sleep(2.2)
+        await asyncio.sleep(2.2)
         await self.wss.send(data)
 
     async def joinVideoChatAsSpectator(self, comId: str, chatId: str):
@@ -484,7 +516,7 @@ class WssClient:
             },
             "t": 112,
         }
-        asyncio.sleep(2.2)
+        await asyncio.sleep(2.2)
         await self.wss.send(data)
 
     async def threadJoin(self, comId: str, chatId: str):
@@ -557,7 +589,7 @@ class WssClient:
             "t": 300,
         }
 
-        asyncio.sleep(2.2)
+        await asyncio.sleep(2.2)
         await self.wss.send(data)
 
         for i in range(3):
@@ -575,9 +607,6 @@ class WssClient:
 
 class Wss(AsyncCallbacks, WssClient, Headers):
     def __init__(self, client, trace: bool = False, is_bot: bool = False):
-        self.ev = AsyncEvents(str_only=False, first_parameter_object=False, default_event=False)
-        self.ev.add_event(callback=self.res, aliases=["res"], event_type="wss")
-
         self.trace = trace
         self.socket = None
         self.headers = None
@@ -591,14 +620,14 @@ class Wss(AsyncCallbacks, WssClient, Headers):
         self.narvi = "https://service.narvii.com/api/v1/"
         self.socket_url = "wss://ws1.narvii.com"
         self.lastMessage = {}
-        websocket.enableTrace(trace)
+        # websocket.enableTrace(trace)
 
-    def onOpen(self, *args):
+    async def onOpen(self, *args):
         self.isOpened = True
         if self.trace:
             print("[ON-OPEN] Sockets are open")
 
-    def onClose(self, *args):
+    async def onClose(self, *args):
         self.isOpened = False
         if self.trace:
             print("[ON-CLOSE] Sockets are closed")
@@ -611,18 +640,14 @@ class Wss(AsyncCallbacks, WssClient, Headers):
             print("[RECEIVE] returning last message")
         return self.lastMessage
 
-    def on_message(self, ws, data):
+    async def on_message(self, ws, data):
         self.lastMessage = json.loads(data)
-        self.ev.add_task(data={"event": "res", "parameters": {"data": data}}, event_type="wss", str_only=False)
-        self.ev.run_task_sync()
+        await self.resolve(data)
 
         if self.trace:
             print("[ON-MESSAGE] Received a message . . .")
 
-    async def res(self, data):
-        await self.resolve(data)
-
-    async def launch(self):
+    async def launch(self, first_time: bool = True):
         final = f"{self.client.deviceId}|{int(timer.time() * 1000)}"
         self.headers = {
             "NDCDEVICEID": self.client.deviceId,
@@ -630,28 +655,65 @@ class Wss(AsyncCallbacks, WssClient, Headers):
             "NDC-MSG-SIG": util.generateSig(data=final),
         }
 
-        self.socket = websocket.WebSocketApp(
+        self.socket = WebSocketApp(
             f"{self.socket_url}/?signbody={final.replace('|', '%7C')}",
             on_message=self.on_message,
             on_close=self.onClose,
             on_open=self.onOpen,
-            header=self.headers,
+            on_error=self.onError,
+            header=self.headers
         )
 
         if self.trace:
             print("[LAUNCH] Sockets starting . . . ")
 
-        threading.Thread(
-            target=self.socket.run_forever, kwargs={"ping_interval": 60}
-        ).start()
+        threading.Thread(target=self.run_socket_forever).start()
+
+        if first_time:
+            loop, th = self.start_async()
+            loop = self.submit_async(self.reboot_socket(), loop)
+
+    async def reboot_socket(self):
+        self.run_reboot = True
+        while self.run_reboot:
+            await asyncio.sleep(300)
+            await self.close()
+            await self.launch(False)
+
+    def start_async(self):
+        loop = asyncio.new_event_loop()
+        th = threading.Thread(target=loop.run_forever)
+        th.start()
+        return loop, th
+
+    def submit_async(self, awaitable, loop):
+        return asyncio.run_coroutine_threadsafe(awaitable, loop)
+
+    def stop_async(self, loop):
+        loop.call_soon_threadsafe(loop.stop)
+
+    def run_socket_forever(self, **kwargs):
+        loop, th = self.start_async()
+        self.wss_loop = self.submit_async(self.socket.run_forever(ping_interval=30), loop)
+        self.th = th
 
     async def close(self):
-        self.socket.close()
+        self.socket._stop_ping_thread()
+        self.socket.keep_running = False
+
+        if self.socket.sock:
+            await self.socket.sock.close()
+
+        self.socket.sock = None
 
         if self.trace:
             print("[CLOSE] closing socket . . .")
 
-        asyncio.sleep(1.5)
+        await asyncio.sleep(1.5)
+
+    async def onError(self, *args):
+        if self.trace:
+            print(f"[ERROR] Error! {args}")
 
     def socket_status(self):
         print("\nSockets are OPEN\n") if self.isOpened else print(
