@@ -135,6 +135,24 @@ class SubClient(Acm, Session):
         )
         return ThreadList(req["threadList"]).ThreadList
 
+    def full_embed(self, link: str, image: BinaryIO, message: str, chatId: str):
+        data = {
+            "type": 0,
+            "content": message,
+            "extensions": {
+                "linkSnippetList": [{
+                    "link": link,
+                    "mediaType": 100,
+                    "mediaUploadValue": b64encode(image.read()).decode(),
+                    "mediaUploadValueContentType": "image/png"
+                }]
+            },
+            "clientRefId": int(timestamp() / 10 % 100000000),
+            "timestamp": int(timestamp() * 1000),
+            "attachedObject": None
+        }
+        return Json(self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}/message", data))
+
     def send_message(
         self,
         chatId: str,
@@ -1121,10 +1139,10 @@ class SubClient(Acm, Session):
 
         req = self.postRequest(f"/x{self.comId}/s/flag", data)
         return Json(req)
-
+#thanks to V¡ktor#9475 for ideas
     def send_active_time(self, tz: int = int(-timezones // 1000), timers: list = None):
         data = {
-            "userActiveTimeChunkList": [timers if timers else {"start": int(timestamp()), "end": int(timestamp() + 300)}],
+            "userActiveTimeChunkList": timers if timers else [{"start": int(timestamp()), "end": int(timestamp() + 300)}],
             "timestamp": int(timestamp() * 1000),
             "optInAdsFlags": 2147483647,
             "timezone": tz,
